"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  generateJSONPatch: () => generateJSONPatch,
  pathInfo: () => pathInfo
});
module.exports = __toCommonJS(src_exports);
function generateJSONPatch(before, after, config = {}) {
  const { objectHash, propertyFilter } = config;
  const patch = [];
  const hasPropertyFilter = typeof propertyFilter === "function";
  function compareArrayByIndex(leftArr, rightArr, path) {
    let currentIndex = 0;
    const maxLength = Math.max(leftArr.length, rightArr.length);
    for (let i = 0; i < maxLength; i++) {
      const newPathIndex = `${path}/${currentIndex++}`;
      if (i < leftArr.length && i < rightArr.length) {
        compareObjects(newPathIndex, leftArr[i], rightArr[i]);
      } else if (i >= leftArr.length && i < rightArr.length) {
        patch.push({ op: "add", path: newPathIndex, value: rightArr[i] });
      } else if (i < leftArr.length && i >= rightArr.length) {
        patch.push({ op: "remove", path: newPathIndex });
        currentIndex--;
      }
    }
  }
  function compareArrayByHash(leftArr, rightArr, path) {
    var _a;
    if (typeof objectHash !== "function") {
      throw Error("No objectHash function provided");
    }
    const leftHashes = leftArr.map((value) => objectHash(value, { side: "left", path }));
    const rightHashes = rightArr.map((value) => objectHash(value, { side: "right", path }));
    let currentIndex = 0;
    const targetHashes = [];
    for (let i = 0; i < leftHashes.length; i++) {
      const newPathIndex = `${path}/${currentIndex++}`;
      const rightHashIndex = rightHashes.indexOf(leftHashes[i]);
      if (rightHashIndex >= 0) {
        compareObjects(newPathIndex, leftArr[i], rightArr[rightHashIndex]);
        targetHashes.push(leftHashes[i]);
      } else {
        patch.push({ op: "remove", path: newPathIndex });
        currentIndex--;
      }
    }
    const toBeAddedHashes = rightHashes.filter((hash) => !targetHashes.includes(hash));
    for (const toBeAddedHash of toBeAddedHashes) {
      patch.push({
        op: "add",
        path: `${path}/${currentIndex++}`,
        value: rightArr[rightHashes.indexOf(toBeAddedHash)]
      });
      targetHashes.push(toBeAddedHash);
    }
    if ((_a = config.array) == null ? void 0 : _a.ignoreMove) {
      return;
    }
    for (let i = rightHashes.length - 1; i >= 0; i--) {
      const hash = rightHashes[i];
      const targetIndex = rightHashes.indexOf(hash);
      const currentIndex2 = targetHashes.indexOf(hash);
      if (currentIndex2 !== targetIndex) {
        patch.push({
          op: "move",
          from: `${path}/${currentIndex2}`,
          path: `${path}/${targetIndex}`
        });
        moveArrayElement(targetHashes, currentIndex2, targetIndex);
      }
    }
  }
  function compareArrays(leftArr, rightArr, path) {
    if (JSON.stringify(leftArr) === JSON.stringify(rightArr))
      return;
    if (objectHash) {
      compareArrayByHash(leftArr, rightArr, path);
    } else {
      compareArrayByIndex(leftArr, rightArr, path);
    }
  }
  function compareObjects(path, leftJsonValue, rightJsonValue) {
    const isArrayAtTop = path === "" && [leftJsonValue, rightJsonValue].every(Array.isArray);
    if (isPrimitiveValue(leftJsonValue) || isPrimitiveValue(rightJsonValue)) {
      if (leftJsonValue !== rightJsonValue) {
        patch.push({ op: "replace", path, value: rightJsonValue });
      }
      return;
    }
    if (isArrayAtTop) {
      return compareArrays(leftJsonValue, rightJsonValue, "");
    }
    if ([leftJsonValue, rightJsonValue].some(Array.isArray)) {
      patch.push({ op: "replace", path, value: rightJsonValue });
      return;
    }
    for (const rightKey in rightJsonValue) {
      if (hasPropertyFilter && !propertyFilter(rightKey, { side: "right", path }))
        continue;
      let newPath = isArrayAtTop && path === "" ? `/${rightKey}` : `${path}/${rightKey}`;
      const leftValue = leftJsonValue[rightKey];
      const rightValue = rightJsonValue[rightKey];
      if (Array.isArray(leftValue) && Array.isArray(rightValue)) {
        compareArrays(leftValue, rightValue, newPath);
      } else if (isJsonObject(rightValue)) {
        if (isJsonObject(leftValue)) {
          compareObjects(newPath, leftValue, rightValue);
        } else {
          patch.push({ op: "replace", path: newPath, value: rightValue });
        }
      } else if (!leftJsonValue.hasOwnProperty(rightKey)) {
        patch.push({ op: "add", path: newPath, value: rightValue });
      } else if (leftValue !== rightValue) {
        patch.push({ op: "replace", path: newPath, value: rightValue });
      }
    }
    for (const leftKey in leftJsonValue) {
      if (!leftJsonValue.hasOwnProperty(leftKey) || hasPropertyFilter && !propertyFilter(leftKey, { side: "left", path }))
        continue;
      if (!rightJsonValue.hasOwnProperty(leftKey)) {
        let newPath = isArrayAtTop && path === "" ? `/${leftKey}` : `${path}/${leftKey}`;
        patch.push({ op: "remove", path: newPath });
      }
    }
  }
  compareObjects("", before, after);
  return [...patch];
}
function isPrimitiveValue(value) {
  return value === void 0 || value === null || typeof value === "undefined" || typeof value === "string" || typeof value === "number" || typeof value === "boolean";
}
function isJsonObject(value) {
  return (value == null ? void 0 : value.constructor) === Object;
}
function moveArrayElement(array, from, to) {
  array.splice(to, 0, array.splice(from, 1)[0]);
}
function pathInfo(path) {
  const segments = path.split("/");
  const length = segments.length;
  const last = segments[length - 1];
  return { segments, length, last };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  generateJSONPatch,
  pathInfo
});
